import libcontext
from libcontext.socketclasses import *
from libcontext.pluginclasses import *

from ..canvas import canvasdrone
from .pandacanvas_bgrid import pandacanvas_bgrid
from .pandacanvas_mstr import pandacanvas_mstr
from .pandacanvas_image import pandacanvas_image
import functools

pandacanvasinstances = set((0,))

class pandacanvas(canvasdrone,
                  pandacanvas_bgrid,
                  pandacanvas_mstr,
                  pandacanvas_image,
                 ): #call place() explicitly for all mixins!
  classname = "pandaclass"
  def __init__(self):
    global pandacanvasinstances
    self.instances = {}
    self.canvasinstance = max(pandacanvasinstances)+1
    pandacanvasinstances.add(self.canvasinstance)
    self._render2d = None
    self._aspect2d = None
    self._pixel2d = None
    self.sortids = set()
    self.next_sortid = 1
  def get_next_sortid(self):
    ret = self.next_sortid
    self.sortids.add(ret)
    self.next_sortid += 1
    return ret
  def _set_render2d(self,get_render2d):
    self.get_render2d = get_render2d
  def _set_aspect2d(self,get_aspect2d):
    self.get_aspect2d = get_aspect2d
  def _set_pixel2d(self,get_pixel2d):
    self.get_pixel2d = get_pixel2d
  
  def get_parent_render2d(self):
    if self._render2d is None:
      self._render2d = self.get_render2d().attachNewNode("render2d-pandacanvas")
      self._render2d.setPos(-1,0,1)
      self._render2d.setScale(2,1,-2)
    return self._render2d

  def get_parent_aspect2d(self):
    if self._aspect2d is None:
      self._aspect2d = self.get_aspect2d().attachNewNode("aspect2d-pandacanvas")
      x,y = self.canvas_size()
      self._aspect2d.setPos(-float(x)/y,0,1)
      self._aspect2d.setScale(2,1,-2)
    return self._aspect2d

  def get_parent_pixel2d(self):
    if self._pixel2d is None:
      self._pixel2d = self.get_pixel2d().attachNewNode("pixel2d-pandacanvas")
      self._pixel2d.setScale(1,1,-1)
    return self._pixel2d
  
  def get_canvas_size(self, canvas_sizefunc):
    self.canvas_size = canvas_sizefunc
  
  def place(self):
    oldplugs = self._get_oldplugins()    
    libcontext.socket(("panda", "noderoot", "pixel2d"), socket_single_required(self._set_pixel2d))
    libcontext.socket(("panda", "noderoot", "render2d"), socket_single_required(self._set_render2d))
    libcontext.socket(("panda", "noderoot", "aspect2d"), socket_single_required(self._set_aspect2d))
    libcontext.socket(("canvas","size"), socket_single_required(self.get_canvas_size))
    
    #super() and stuff doesn't work, call place() explicitly for each mixin
    pandacanvas_bgrid.place(self)
    pandacanvas_mstr.place(self)
    pandacanvas_image.place(self)

    #finally, call canvasdrone.place
    canvasdrone._wrapped_hive.place(self,oldplugs)
